/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("panel.index");

/** CONSTANT: Number of allowed characters before truncating a string. */
BookmarkTools.panel.truncateNum = 60;

/**
	Basic panel namespace that provides functions to control settings
	and widgets in the configuration pane.
	@namespace
*/
BookmarkTools.panel.index = {
	
	init : function(){ 
		try{
			/** general settings, compatible with all build modes */
			BookmarkTools.panel.errors.loadErrorPrefs(); 
			BookmarkTools.panel.duplicates.loadDuplicatePrefs(); 
			
			var errorType = BookmarkTools.utils.Prefs.getInt('errorType');
			BookmarkTools.$("bookmarkTools.pane.tab.http").setAttribute("selected", (errorType == 0));
			BookmarkTools.$("bookmarkTools.pane.tab.other").setAttribute("selected", (errorType == 1));
			BookmarkTools.$("bookmarkTools.pane.tab.display").setAttribute("selectedIndex", errorType);
			
			this.startLookup(); 
		
			// widgets
			var buildMode = BookmarkTools.$("bookmarkTools.pane.radiogroupBT").selectedItem.id;
			var intervalSlider = BookmarkTools.$("bookmarkTools.pane.intervalSlider");
			var buildButton = BookmarkTools.$("bookmarkTools.pane.buildButton");
			var deleteButton = BookmarkTools.$("bookmarkTools.pane.deleteButton");
		
			var checkButton = BookmarkTools.$("bookmarkTools.pane.checkButton");
			var discardErrorsButton = BookmarkTools.$("bookmarkTools.pane.discardErrorButton");
			var discardDuplicatesButton = BookmarkTools.$("bookmarkTools.pane.discardDuplicateButton");

			/** automatic build mode */
			if(buildMode == "bookmarkTools.pane.automatic"){
				/** only works when indexer is running */
				if(BookmarkTools.utils.Prefs.getBool('isRunning')){
					intervalSlider.disabled = true;
					BookmarkTools.$("bookmarkTools_pane_loadFolder").disabled = true;
					BookmarkTools.$("bookmarkTools.pane.loadFolderCheck").disabled = true;
					BookmarkTools.$("bookmarkTools.pane.radiogroupBT").disabled = true;
				}else { /** else */
					intervalSlider.disabled = false; 
				}
				// always
				buildButton.disabled = true; 
				deleteButton.disabled = true; 
				checkButton.disabled = true;
				discardErrorsButton.disabled = true;
				discardDuplicatesButton.disabled = true;
			}
			
			/** manual build mode */
			if(buildMode == "bookmarkTools.pane.manual"){
				intervalSlider.disabled = true; 
				buildButton.disabled = false;
				deleteButton.disabled = false;
				checkButton.disabled = false;
				discardErrorsButton.disabled = false;
				discardDuplicatesButton.disabled = false;
			}
			
			// load preferences
			BookmarkTools.$("bookmarkTools.pane.intervalSlider").value = BookmarkTools.utils.Prefs.getInt("indexDelay");
			var path = BookmarkTools.utils.Prefs.getFile("indexPath").path;
			BookmarkTools.$("bookmarkTools.pane.pathTooltip").firstChild.value = path;
			BookmarkTools.$("bookmarkTools.pane.pathInfo").value = BookmarkTools.utils.StringUtils.truncate(path, BookmarkTools.panel.truncateNum, ' [..]');
			BookmarkTools.$("bookmarkTools_pane_loadFolder").disabled = BookmarkTools.utils.Prefs.getBool("indexPathDefault");
		}catch(e){
			BookmarkTools.reportError(e);
		}
	},
	
	uninit : function(){
		// set slider preference
		BookmarkTools.utils.Prefs.setInt("indexDelay", BookmarkTools.$("bookmarkTools.pane.intervalSlider").value);
	},
	
	showRestartMessage : function(item){
		var currentBuildType = BookmarkTools.utils.Prefs.getInt('buildType');
		if(BookmarkTools.utils.NumberUtils.parseInt(item.value) != currentBuildType){
			BookmarkTools.$('bookmarkTools.restartMsg').setAttribute("style", "color: red; visibility: visible;");
		}else {
			BookmarkTools.$('bookmarkTools.restartMsg').setAttribute("style", "visibility: hidden;");
		}
		this.setBuildTypePrefs(item);
	},
	
	setBuildTypePrefs : function(item){
		if(item.id == 'bookmarkTools.pane.automatic')
			BookmarkTools.utils.Prefs.setBool('firstRun', true);
	},
	
	setLocationValue : function(slider){
		var elem = BookmarkTools.$("bookmarkTools.pane.info");
		var val = elem.value.split(" ")[1];
		elem.value = slider.value + " " + val;
	},
	
	setUserDefinedPath : function(checkbox){
		if(!checkbox.checked){
			BookmarkTools.$("bookmarkTools_pane_loadFolder").disabled = true;
			BookmarkTools.utils.Prefs.remove("alreadyAddedBookmarks");
		}else {
			BookmarkTools.$("bookmarkTools_pane_loadFolder").disabled = false;
		}
	},
	
	setIndexPath : function(){
		var nsIFilePicker = Ci.nsIFilePicker;
		var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
		fp.init(window, "Select folder", nsIFilePicker.modeGetFolder);
		var res = fp.show();
		if (res == nsIFilePicker.returnOK){
			var path = fp.file.path;
			BookmarkTools.utils.Prefs.setFile("indexPath", fp.file);
			BookmarkTools.$("bookmarkTools.pane.pathTooltip").firstChild.value = path;
			BookmarkTools.$("bookmarkTools.pane.pathInfo").value = BookmarkTools.utils.StringUtils.truncate(path, BookmarkTools.panel.truncateNum, ' [..]');
			BookmarkTools.utils.Prefs.remove("alreadyAddedBookmarks");
		}
	},
	
	/**
		Shows currently indexed bookmarks with their status.
		
		Note:
			Never change indexed bookmarks while lookup, because the delete function 
			receives tree indexes that are used to refer to the corresponding document.
	*/
	startLookup : function(){
		try {
			var array = BookmarkTools.indexing.Index._lookup();
			var e = array.enumerate();
	
			var errors = BookmarkTools.utils.Prefs.getBranch('invBm').toHash();
			var tree_indexed = BookmarkTools.$("bookmarkTools.pane.indexTree");
			tree_indexed.clear();

			var untraceableBookmarks = new BookmarkTools.dataTypes.ExtendedHash();
			var pos = 0, lookupNode, status;
			
			while (e.hasMoreElements()) {
				lookupNode = e.getNext().QueryInterface(Ci.ILookupNode);

				status = BookmarkTools.indexing.STATUS_SUCCESS; 
				if(!BookmarkTools.common.Book.isBookmarked(lookupNode.primaryKey)){
					status = BookmarkTools.indexing.STATUS_UNTRACEABLE;
					untraceableBookmarks.put(lookupNode.primaryKey, pos);
					
					if(BookmarkTools.utils.Prefs.getBool('cleanupUntraceableBookmarks')){
						pos++;
						continue;
					}
				}else if(errors.contains(lookupNode.primaryKey)){
					status = BookmarkTools.indexing.STATUS_CORRUPTED;
				}

				tree_indexed.add(lookupNode.name, lookupNode.primaryKey, status, lookupNode.language);
				pos++;
			}
			
			//checks preferences if cleanup flag is set
			//and removes all untraceable bookmarks
			this.doCleanup(untraceableBookmarks);
			
			BookmarkTools.$("bookmarkTools.pane.indexCtr").value = "(" + tree_indexed.size + ")";
		} catch(e){
			BookmarkTools.reportError(e);
		}
	},
	
	doCleanup : function(untraceableBookmarks){
		if(BookmarkTools.utils.Prefs.getBool('cleanupUntraceableBookmarks') && !untraceableBookmarks.isEmpty()){
			BookmarkTools.indexing.Index._delete(untraceableBookmarks);
		}
	},
	
	/**
		Runs a routine to check all bookmarks against http errors or duplicates.
	*/
	doCheck : function(){
		//clear prefs
		BookmarkTools.utils.Prefs.getBranch('invBm').clear();
		BookmarkTools.utils.Prefs.getBranch('dupBm').clear();
		
		//clear last log files
		BookmarkTools.utils.FileIO.clearLogs();
		
		//clear tree and button information
		BookmarkTools.$("bookmarkTools.pane.errorTree").clear();
		BookmarkTools.$("bookmarkTools.pane.errorCtr").value = "(0)";
		
		BookmarkTools.$("bookmarkTools.pane.otherTree").clear();
		BookmarkTools.$("bookmarkTools.pane.otherCtr").value = "(0)";
		
		BookmarkTools.$("bookmarkTools.pane.duplicateTree").clear();
		BookmarkTools.$("bookmarkTools.pane.duplicateCtr").value = "(0)";
		
		//init crawler
		BookmarkTools.indexing.Crawler.init();
		BookmarkTools.indexing.Crawler.setBookmarks(BookmarkTools.common.Book.getResourceHash());

		//init filter
		BookmarkTools.indexing.Filter.init();
		BookmarkTools.indexing.Filter.setCrawler(BookmarkTools.indexing.Crawler);
		BookmarkTools.indexing.Filter.setCheck(true);
		BookmarkTools.indexing.Filter.setView(BookmarkTools.indexing.View);
		
		BookmarkTools.indexing.Index.setMode(BookmarkTools.indexing.MANUAL);
		
		var retVals = {filter : BookmarkTools.indexing.Filter};
		openDialog("chrome://bookmarktools/content/bookmarkToolsProgress.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", retVals);
	},
	
	/**
		Runs a routine to index bookmarks manually.
	*/
	doAdd : function(){
		//clear prefs
		BookmarkTools.utils.Prefs.getBranch('invBm').clear();
		BookmarkTools.utils.Prefs.getBranch('dupBm').clear();
		
		//clear last log files
		BookmarkTools.utils.FileIO.clearLogs();
		
		//clear tree and button information
		BookmarkTools.$("bookmarkTools.pane.errorTree").clear();
		BookmarkTools.$("bookmarkTools.pane.errorCtr").value = "(0)";
		
		BookmarkTools.$("bookmarkTools.pane.indexTree").clear();
		BookmarkTools.$("bookmarkTools.pane.indexCtr").value = "(0)";
		
		BookmarkTools.$("bookmarkTools.pane.otherTree").clear();
		BookmarkTools.$("bookmarkTools.pane.otherCtr").value = "(0)";
		
		BookmarkTools.$("bookmarkTools.pane.duplicateTree").clear();
		BookmarkTools.$("bookmarkTools.pane.duplicateCtr").value = "(0)";
		
		//clear search dir
		BookmarkTools.utils.FileIO.clear(BookmarkTools.utils.FileIO.getSearchDir());
		
		var bookmarks = BookmarkTools.common.Book.getResourceHash();
		
		//init crawler
		BookmarkTools.indexing.Crawler.init();
		BookmarkTools.indexing.Crawler.setBookmarks(bookmarks);

		//init filter
		BookmarkTools.indexing.Filter.init();

		//bind with index
		BookmarkTools.indexing.Index.setMode(BookmarkTools.indexing.MANUAL);
		BookmarkTools.indexing.Index.setFilter(BookmarkTools.indexing.Filter);

		BookmarkTools.indexing.Filter.setCrawler(BookmarkTools.indexing.Crawler);
		BookmarkTools.indexing.Filter.setView(BookmarkTools.indexing.View);

		var retVals = {index : BookmarkTools.indexing.Index};
		openDialog("chrome://bookmarktools/content/bookmarkToolsProgress.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", retVals);
	},
	
	/**
		Opens an information window.
	*/
	doInfo : function(){
		var args = {
			bookmarks: BookmarkTools.common.Book.getSize(),
			diskSize: BookmarkTools.utils.FileIO.formatBytes(BookmarkTools.utils.FileIO.getDirectoryInfo(BookmarkTools.utils.FileIO.getSearchDir())),
			numberOfDocuments: BookmarkTools.indexing.Index._size(),
			buildDate: (1 * BookmarkTools.utils.Prefs.getChar('intIndexDate')),
			delay: BookmarkTools.indexing.Index.delay,
			mode: BookmarkTools.indexing.Index.mode
		};
		openDialog("chrome://bookmarktools/content/bookmarkToolsInfo.xul", "_blank", "chrome, centerscreen, modal, resizable=no, titlebar=no", args);
	},
	
	/**
		Runs a routine thats checks if the current time is valid for automatic index
		rebuild. If not, nothing happens, else the indexing process starts.
	*/
	tryBuildAutomatic : function(){ 

		//checks if the build type 'AUTOMATIC' is set and 
		//if the current time is greater then the last build time + delay
		if(BookmarkTools.indexing.Index.isValidBuildTime()){
			
			if(BookmarkTools.globals.lockKey != -1){
			
				if(BookmarkTools.utils.Prefs.getBool('firstRun')){ //first run?
					//clear prefs
					BookmarkTools.utils.Prefs.getBranch('invBm').clear();
					BookmarkTools.utils.Prefs.getBranch('dupBm').clear();
					
					//clear last log files
					BookmarkTools.utils.FileIO.clearLogs();
					
					//clear build directory
					BookmarkTools.utils.FileIO.clear(BookmarkTools.utils.FileIO.getBuildDir());
					
					//generate log file
					var file = BookmarkTools.utils.FileIO.generateFile(BookmarkTools.utils.FileIO.getBuildDir(), "alreadyAddedBookmarks.log");
					BookmarkTools.utils.Prefs.setFile("alreadyAddedBookmarks", file);
				}
			
				//init crawler
				BookmarkTools.indexing.Crawler.init();

				//init filter
				BookmarkTools.indexing.Filter.init();
				var indexState = BookmarkTools.indexing.Filter.getBookmarksFromFile();
				BookmarkTools.indexing.Crawler.setBookmarks(indexState.remainingBookmarks);

				//bind with index
				BookmarkTools.indexing.Index.setMode(BookmarkTools.indexing.AUTOMATIC);
				BookmarkTools.indexing.Index.setFilter(BookmarkTools.indexing.Filter);
				
				BookmarkTools.indexing.Filter.setCrawler(
					BookmarkTools.indexing.Crawler, 
					indexState.currentIndexed, 
					indexState.allBookmarks
				);
				
				BookmarkTools.indexing.BrowserView.init();
				BookmarkTools.indexing.Filter.setView(BookmarkTools.indexing.BrowserView);
				
				BookmarkTools.indexing.Index._build();
			
				//sets first run to false
				BookmarkTools.utils.Prefs.setBool('firstRun', false);
				BookmarkTools.utils.Prefs.setBool('isRunning', true);
			}
		}
	},
	
	/**
		Deletes all selected bookmarks from index data. If nothing is selected all the whole
		index is removed.
	*/
	doDelete : function(){
		var tree = BookmarkTools.$('bookmarkTools.pane.indexTree');
		var indexes = tree.selection;
		tree.unselect();
		
		//set path to search directory
		BookmarkTools.indexing.Index.setMode(BookmarkTools.indexing.MANUAL);
		BookmarkTools.indexing.Index._delete(indexes);

		this.startLookup();
	}
	
};
 