/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("indexing");

/**
	Pattern to detect whitespaces like /\t /\n /\f /\v.
*/
BookmarkTools.indexing.PATTERN_WHITESPACES = /(\s)+/gi;

/**
	Pattern to detect digits [0-9].
*/
BookmarkTools.indexing.PATTERN_NUMBERS = /[0-9]+/gi;

/**
	Pattern to detect quotes like " or '.
*/
BookmarkTools.indexing.PATTERN_QUOTES = /["']+/gi;

/**
	Pattern to detect entities.
*/
BookmarkTools.indexing.PATTERN_ENTITIES = /&([a-zA-Z0-9#]+);/gi;

/**
	Pattern to detect http mimetypes.
*/
BookmarkTools.indexing.PATTERN_MIMETYPE = /(application|text)\/[a-zA-Z0-9-+.]+/gi;

/**
	Pattern to detect file endings.
*/
BookmarkTools.indexing.PATTERN_FILE_ENDING = /\.[a-zA-Z]+$/gi;

/**
	Pattern to validate urls.
*/
BookmarkTools.indexing.PATTERN_TOK_URL = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?$/;

/**
	Pattern to validate id adresses.
*/
BookmarkTools.indexing.PATTERN_TOK_IP_ADRESS = "(?:(?:25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])";

/**
	Pattern to validate email expressions.
*/
BookmarkTools.indexing.PATTERN_TOK_EMAIL = /^[a-zA-Z0-9._-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$/;

/**
	Pattern to detect marks.
*/
BookmarkTools.indexing.PATTERN_MARKS = /[!"$%&\/()=?\`\'*\+#-;:\<\>@\[\]\{\}\^\~_\|\.,]+/gi;

/**
	Pattern to detect marks.
*/
BookmarkTools.indexing.PATTERN_MARKS_LAZY = /[!"$%&\/()=?\`\'*\+;:\<\>@\[\]\{\}\^\~_\|\.,]+/gi;

/**
	Pattern to detect html color expressions like #ff0000.
*/
BookmarkTools.indexing.PATTERN_CSS_COLOR = /#[a-f0-9]{6}/gi;

/**
	Pattern to detect css unit expressions like 12px or 1.5em.
*/
BookmarkTools.indexing.PATTERN_CSS_UNIT = /([+-]?[0-9]+)(px|pt|cm|mm|in|pc|em|ex)/gi;

/**
	Pattern to detect lang attributes which specifies the content language.
	@see HTML 4.0 specification at <a href="http://www.w3.org/TR/REC-html40/">http://www.w3.org/TR/REC-html40/</a>
*/
BookmarkTools.indexing.PATTERN_ATTR_LANG = /\slang\s*=\s*[\"'][a-zA-Z-]+[\"']/gi;

/**
	Pattern to detect name attributes.
*/
BookmarkTools.indexing.PATTERN_ATTR_NAME = /\sname\s*=\s*[\"'].+[\"']\s/gi;

/**
	Pattern to detect meta tags with name="title".
*/
BookmarkTools.indexing.PATTERN_ATTR_NAME_TITLE = /\sname\s*=\s*[\"']?title[\"']?\s/gi;

/**
	Pattern to detect meta tags with name="description".
*/
BookmarkTools.indexing.PATTERN_ATTR_NAME_DESC = /\sname\s*=\s*[\"']?description[\"']?\s/gi;

/**
	Pattern to detect meta tags with name="keywords".
*/
BookmarkTools.indexing.PATTERN_ATTR_NAME_KEYS = /\sname\s*=\s*[\"']?keywords[\"']?\s/gi;

/**
	Pattern to detect meta tags with name="language".
*/
BookmarkTools.indexing.PATTERN_ATTR_NAME_LANG = /\sname\s*=\s*[\"']?language[\"']?\s/gi;

/**
	Pattern to detect meta tags with name="content-language".
*/
BookmarkTools.indexing.PATTERN_ATTR_NAME_LANG_2 = /\sname\s*=\s*[\"']?content-language[\"']?\s/gi;

/**
	Pattern to detect href attributes.
*/
BookmarkTools.indexing.PATTERN_ATTR_HREF = /href[\s]*=[\s]*["'].+["']/gi;

/**
	Pattern to detect content attributes.
*/
BookmarkTools.indexing.PATTERN_ATTR_CONTENT = /content\s*=\s*[\"'][a-zA-Z-,]+[\"']/gi;

/**
	Pattern to charset href attributes.
*/
BookmarkTools.indexing.PATTERN_ATTR_CHAR_SET = /charset=[a-zA-Z0-9-_]+/gi;

/**
	Pattern to detect http-equiv attributes.
*/
BookmarkTools.indexing.PATTERN_ATTR_HTTP_EQUIV = /http-equiv\s*=\s*[\"']content-language[\"']/gi;

/**
	Pattern to detect encoding attributes.
*/
BookmarkTools.indexing.PATTERN_ATTR_ENCODING = /encoding[\s]*=[\s]*"[a-zA-Z0-9-]*"/gi;

/**
	Pattern to detect the xml initial line (e.g. <?xml version="1.0"?>).
*/
BookmarkTools.indexing.PATTERN_TAG_XML = /<\?xml.*\?>/gi;

/**
	Pattern to detect the doctype declaration.
*/
BookmarkTools.indexing.PATTERN_TAG_DOCTYPE = /<\!doctype.*>/gi;

/**
	Pattern to detect meta tags (tries to ignore tag validation)
*/
BookmarkTools.indexing.PATTERN_TAG_META = /<meta\/?[^>]+>/gi;

/**
	Pattern to detect script declarations.
*/
BookmarkTools.indexing.PATTERN_TAG_SCRIPT = "<script[^>]*>([\\S\\s]*?)<\/script>";

/**
	Pattern to detect any tag declarations.
*/
BookmarkTools.indexing.PATTERN_TAG_ANY = /<\w+(\s+("[^"]*"|'[^']*'|[^>])+)?>|<\/\w+>/gi;

/**
	Pattern to detect ankers.
*/
BookmarkTools.indexing.PATTERN_TAG_A = /<a[^>]*>.*<\/a>/gi;
